SystemOrganization addCategory: #'WebClient-Monticello'!

!WebRequest methodsFor: '*webclient-monticello-named-responses' stamp: 'SvenVanCaekenberghe 6/28/2010 17:12'!
sendBadRequest
	^ self sendResponseCode: 400! !

!WebRequest methodsFor: '*webclient-monticello-named-responses' stamp: 'SvenVanCaekenberghe 6/28/2010 17:09'!
sendCreated
	^ self sendCreated: self rawUrl ! !

!WebRequest methodsFor: '*webclient-monticello-named-responses' stamp: 'SvenVanCaekenberghe 6/28/2010 17:11'!
sendCreated: url
	^ self 
		sendResponseCode: 201
		contents: 'Successfully created URL ', url
		do: [ :response | response headerAt: 'Location' put: url ]! !

!WebRequest methodsFor: '*webclient-monticello-named-responses' stamp: 'SvenVanCaekenberghe 6/28/2010 17:13'!
sendNotAcceptable
	^ self sendResponseCode: 406! !

!WebRequest methodsFor: '*webclient-monticello-named-responses' stamp: 'SvenVanCaekenberghe 6/28/2010 17:18'!
sendNotFound
	^ self sendNotFound: self rawUrl ! !

!WebRequest methodsFor: '*webclient-monticello-named-responses' stamp: 'SvenVanCaekenberghe 6/28/2010 19:39'!
sendNotFound: url
	^ self 
		sendResponseCode: 404
		contents: 'The requested URL ', url, ' was not found on this server'! !

!WebRequest methodsFor: '*webclient-monticello-coded-responses' stamp: 'SvenVanCaekenberghe 6/28/2010 16:55'!
sendResponseCode: code
	^ self sendResponseCode: code contents: nil do: nil! !

!WebRequest methodsFor: '*webclient-monticello-coded-responses' stamp: 'SvenVanCaekenberghe 6/28/2010 16:55'!
sendResponseCode: code contents: string
	^ self sendResponseCode: code contents: string do: nil! !

!WebRequest methodsFor: '*webclient-monticello-coded-responses' stamp: 'SvenVanCaekenberghe 6/28/2010 18:18'!
sendResponseCode: code contents: string do: block
	"Send a response based on a numeric HTTP status code, optionally add an extra string, 
	or use a block to customize the response, for example its headers" 
	
	| response body |
	response := self newResponse protocol: 'HTTP/1.1' code: code.
	response headerAt: 'Content-Type' put: 'text/html'.
	block ifNotNil: [ block value: response ].
	body := String streamContents: [ :str |
		str nextPutAll: '<html><head><title>'; nextPutAll: response status; nextPutAll: '</title></head>'.
		str nextPutAll: '<body><h1>', response status, '</h1>'.
		string ifNotNil: [ str nextPutAll: '<p>'; nextPutAll: string; nextPutAll: '</p>' ].
		str nextPutAll: '</body></html>' ].
	^ self sendResponse: response content: body! !

!WebRequest methodsFor: '*webclient-monticello-coded-responses' stamp: 'SvenVanCaekenberghe 6/28/2010 16:56'!
sendResponseCode: code do: block
	^ self sendResponseCode: code contents: nil do: block! !

Object subclass: #MCWebServer
	instanceVariableNames: 'webServer directory isAuthenticated'
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'WebClient-Monticello'!

!MCWebServer commentStamp: 'sn 6/2/2012 23:55' prior: 0!
I am MCWebServer.
I implement an HTTP server specialized for the Monticello protocol.

(MCWebServer reset default)
	directory: (FileDirectory  on: (FileDirectory default pathFromURI: 'code'));
	addUser: 'nsuslovi' withPassword: '';
	listenOn: 8800.

Instance Variables:
	webServer <WebServer>
	directory <FileDirectory>
	isAuthenticated <Boolean>!

!MCWebServer class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 11:22'!
default
	Default ifNil: [ Default := self new ].
	^ Default! !

!MCWebServer class methodsFor: 'as yet unclassified' stamp: 'sn 3/8/2013 17:46'!
makeCodeServer

	(MCWebServer reset default)
	directory: (FileDirectory  on: (FileDirectory default pathFromURI: 'code'));
	addUser: '' withPassword: '';
	listenOn: 8800.
! !

!MCWebServer class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 12:08'!
reset
	Default ifNotNil: [ Default destroy. Default := nil ].! !

!MCWebServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 14:44'!
addUser: user withPassword: password
	self isAuthenticated: true.
	self webServer 
		passwordAt: user 
		realm: self realm 
		put: password ! !

!MCWebServer methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/22/2010 11:28'!
destroy
	self webServer destroy ! !

!MCWebServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 11:27'!
directory
	^ directory! !

!MCWebServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 14:44'!
directory: aFileDirectory
	directory := aFileDirectory! !

!MCWebServer methodsFor: 'private' stamp: 'sn 11/2/2012 12:13'!
dispatchWebRequest: request
	request isGetRequest ifTrue: [
		^ request url = '/'
			ifTrue: [ self handleListMczEntries: request ]
			ifFalse: [ self handleGetMczEntry: request ] ].
		
	request isPutRequest ifTrue: [
		self isAuthenticated 
		ifTrue: [ self webServer
			authenticate: request 
			realm: self realm 
			methods: #(basic) 
			do: [ ^self handlePutMczEntry: request ] ]
		ifFalse: [^self handlePutMczEntry: request ]
		
		]! !

!MCWebServer methodsFor: 'private' stamp: 'sn 6/3/2012 01:50'!
handleGetMczEntry: request
	| name |
	name := request url allButFirst.
	(((self isValidMczName: name) or:[(self isValidMcmName: name)]) and: [ self directory fileExists: name ]) 
		ifTrue: [
			self directory readOnlyFileNamed: name do: [ :file | | response |
				
				(self isValidMczName: name) ifTrue:[
				file binary.
				response := request newResponse protocol: 'HTTP/1.1' code: 200.
				response headerAt: 'Content-Type' put: 'application/octet-stream'.
				request sendResponse: response contentStream: file size: file size.].
				
				(self isValidMcmName: name) ifTrue: [
					file ascii.
					response := request newResponse protocol: 'HTTP/1.1' code: 200.
				response headerAt: 'Content-Type' put: 'application/text'.
				request sendResponse: response contentStream: file size: file size.
					
					].
			
			
			] ]
		ifFalse: [ request sendNotFound ]! !

!MCWebServer methodsFor: 'private' stamp: 'sn 11/2/2012 12:39'!
handleListMczEntries: request
	
	request send200Response: self repositoryListing contentType: 'text/html'! !

!MCWebServer methodsFor: 'private' stamp: 'sn 6/3/2012 01:32'!
handlePutMczEntry: request
	| name |
	name := request url allButFirst.
	(((self isValidMczName: name) or:[(self isValidMcmName: name)]) and: [ (self directory fileExists: name) not ]) 
		ifTrue: [
			self directory newFileNamed: name do: [ :file | | response |
				file binary.
				request streamFrom: request stream to: file size: request contentLength progress: nil.
				request sendCreated ] ]
		ifFalse: [ request sendBadRequest ]! !

!MCWebServer methodsFor: 'private' stamp: 'sn 6/3/2012 01:09'!
handleWebRequest: request

self dispatchWebRequest: request
	"self isAuthenticated 
		ifTrue: [ self webServer
			authenticate: request 
			realm: self realm 
			methods: #(basic) 
			do: [ self dispatchWebRequest: request ] ]
		ifFalse: [self dispatchWebRequest: request ]"! !

!MCWebServer methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/22/2010 14:19'!
initialize 
	self webServer: WebServer new.
	self isAuthenticated: false.
	self directory: FileDirectory default.
	self webServer
		addService: '/'
		action: [ :request | self handleWebRequest: request ]
		methods: #(GET PUT)
! !

!MCWebServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 14:18'!
isAuthenticated
	^ isAuthenticated! !

!MCWebServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 14:45'!
isAuthenticated: aBoolean
	isAuthenticated := aBoolean! !

!MCWebServer methodsFor: 'private' stamp: 'sn 6/3/2012 01:31'!
isValidMcmName: string
	^ string isString 
		and: [ string isEmpty not
			and: [ (string endsWith: '.mcm')
				and: [ (string includes: $/) not ] ] ]! !

!MCWebServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/22/2010 13:51'!
isValidMczName: string
	^ string isString 
		and: [ string isEmpty not
			and: [ (string endsWith: '.mcz')
				and: [ (string includes: $/) not ] ] ]! !

!MCWebServer methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/22/2010 12:11'!
listenOn: port
	self webServer listenOn: port! !

!MCWebServer methodsFor: 'as yet unclassified' stamp: 'sn 6/3/2012 01:42'!
mcmEntries

	^ (self directory fileNames select: [ :each | each endsWith: '.mcm' ])
		asSortedCollection! !

!MCWebServer methodsFor: 'accessing' stamp: 'sn 6/3/2012 01:43'!
mczEntries

	^ (self directory fileNames select: [ :each | each endsWith: '.mcz' ])
		asSortedCollection! !

!MCWebServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 11:45'!
realm
	^ 'Monticello Repository'! !

!MCWebServer methodsFor: 'private' stamp: 'sn 6/3/2012 01:43'!
repositoryListing
	^ String streamContents: [ :str |
		str nextPutAll: '<html><head><title>'; nextPutAll: self realm; nextPutAll: '</title></head>'.
		str nextPutAll: '<body><h1>'; nextPutAll: self realm; nextPutAll: '</h1>'.
		str nextPutAll: '<ul>'.
		self mczEntries do: [ :each | 
			str nextPutAll: '<li><a href="'; nextPutAll: each; nextPutAll: '">'; nextPutAll: each; nextPutAll: '</a></li>'].
		str nextPutAll: '</ul></body></html>'.
		self mcmEntries do: [ :each | 
			str nextPutAll: '<li><a href="'; nextPutAll: each; nextPutAll: '">'; nextPutAll: each; nextPutAll: '</a></li>'].
		str nextPutAll: '</ul></body></html>'
		 ]! !

!MCWebServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 11:26'!
webServer
	^ webServer! !

!MCWebServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 14:45'!
webServer: aWebServer
	webServer := aWebServer! !

Object subclass: #MCWebServerSec
	instanceVariableNames: 'webServer directory isAuthenticated'
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'WebClient-Monticello'!

!MCWebServerSec class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 11:22'!
default
	Default ifNil: [ Default := self new ].
	^ Default! !

!MCWebServerSec class methodsFor: 'as yet unclassified' stamp: 'sn 3/8/2013 17:46'!
makeCodeServer

	(MCWebServerSec reset default)
	directory: (FileDirectory  on: (FileDirectory default pathFromURI: 'code2'));
	addUser: '' withPassword: '';
	listenOn: 8801.
! !

!MCWebServerSec class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 12:08'!
reset
	Default ifNotNil: [ Default destroy. Default := nil ].! !

!MCWebServerSec methodsFor: 'accessing' stamp: 'sn 11/2/2012 13:08'!
addUser: user withPassword: password
	self isAuthenticated: true.
	self webServer 
		passwordAt: user 
		realm: self realm 
		put: password ! !

!MCWebServerSec methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/22/2010 11:28'!
destroy
	self webServer destroy ! !

!MCWebServerSec methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 11:27'!
directory
	^ directory! !

!MCWebServerSec methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 14:44'!
directory: aFileDirectory
	directory := aFileDirectory! !

!MCWebServerSec methodsFor: 'private' stamp: 'sn 11/2/2012 13:29'!
dispatchGetRequest: request

		^ request url = '/'
			ifTrue: [ self handleListMczEntries: request ]
			ifFalse: [ self handleGetMczEntry: request ] ! !

!MCWebServerSec methodsFor: 'private' stamp: 'sn 11/2/2012 13:30'!
dispatchWebRequest: request
	
	
	request isGetRequest ifTrue: [
		 self webServer
			authenticate: request 
			realm: self realm 
			methods: #(basic) 
			do: [ 
				
				^self dispatchGetRequest: request.
			
			 ] ].
		
		

		
	request isPutRequest ifTrue: [
	 self webServer
			authenticate: request 
			realm: self realm 
			methods: #(basic) 
			do: [ ^self handlePutMczEntry: request ] ]
		ifFalse: [^self handlePutMczEntry: request ]
		
	! !

!MCWebServerSec methodsFor: 'private' stamp: 'sn 6/3/2012 01:50'!
handleGetMczEntry: request
	| name |
	name := request url allButFirst.
	(((self isValidMczName: name) or:[(self isValidMcmName: name)]) and: [ self directory fileExists: name ]) 
		ifTrue: [
			self directory readOnlyFileNamed: name do: [ :file | | response |
				
				(self isValidMczName: name) ifTrue:[
				file binary.
				response := request newResponse protocol: 'HTTP/1.1' code: 200.
				response headerAt: 'Content-Type' put: 'application/octet-stream'.
				request sendResponse: response contentStream: file size: file size.].
				
				(self isValidMcmName: name) ifTrue: [
					file ascii.
					response := request newResponse protocol: 'HTTP/1.1' code: 200.
				response headerAt: 'Content-Type' put: 'application/text'.
				request sendResponse: response contentStream: file size: file size.
					
					].
			
			
			] ]
		ifFalse: [ request sendNotFound ]! !

!MCWebServerSec methodsFor: 'private' stamp: 'sn 11/2/2012 12:39'!
handleListMczEntries: request
	
	request send200Response: self repositoryListing contentType: 'text/html'! !

!MCWebServerSec methodsFor: 'private' stamp: 'sn 6/3/2012 01:32'!
handlePutMczEntry: request
	| name |
	name := request url allButFirst.
	(((self isValidMczName: name) or:[(self isValidMcmName: name)]) and: [ (self directory fileExists: name) not ]) 
		ifTrue: [
			self directory newFileNamed: name do: [ :file | | response |
				file binary.
				request streamFrom: request stream to: file size: request contentLength progress: nil.
				request sendCreated ] ]
		ifFalse: [ request sendBadRequest ]! !

!MCWebServerSec methodsFor: 'private' stamp: 'sn 6/3/2012 01:09'!
handleWebRequest: request

self dispatchWebRequest: request
	"self isAuthenticated 
		ifTrue: [ self webServer
			authenticate: request 
			realm: self realm 
			methods: #(basic) 
			do: [ self dispatchWebRequest: request ] ]
		ifFalse: [self dispatchWebRequest: request ]"! !

!MCWebServerSec methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/22/2010 14:19'!
initialize 
	self webServer: WebServer new.
	self isAuthenticated: false.
	self directory: FileDirectory default.
	self webServer
		addService: '/'
		action: [ :request | self handleWebRequest: request ]
		methods: #(GET PUT)
! !

!MCWebServerSec methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 14:18'!
isAuthenticated
	^ isAuthenticated! !

!MCWebServerSec methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 14:45'!
isAuthenticated: aBoolean
	isAuthenticated := aBoolean! !

!MCWebServerSec methodsFor: 'private' stamp: 'sn 6/3/2012 01:31'!
isValidMcmName: string
	^ string isString 
		and: [ string isEmpty not
			and: [ (string endsWith: '.mcm')
				and: [ (string includes: $/) not ] ] ]! !

!MCWebServerSec methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/22/2010 13:51'!
isValidMczName: string
	^ string isString 
		and: [ string isEmpty not
			and: [ (string endsWith: '.mcz')
				and: [ (string includes: $/) not ] ] ]! !

!MCWebServerSec methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/22/2010 12:11'!
listenOn: port
	self webServer listenOn: port! !

!MCWebServerSec methodsFor: 'as yet unclassified' stamp: 'sn 6/3/2012 01:42'!
mcmEntries

	^ (self directory fileNames select: [ :each | each endsWith: '.mcm' ])
		asSortedCollection! !

!MCWebServerSec methodsFor: 'accessing' stamp: 'sn 6/3/2012 01:43'!
mczEntries

	^ (self directory fileNames select: [ :each | each endsWith: '.mcz' ])
		asSortedCollection! !

!MCWebServerSec methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 11:45'!
realm
	^ 'Monticello Repository'! !

!MCWebServerSec methodsFor: 'private' stamp: 'sn 6/3/2012 01:43'!
repositoryListing
	^ String streamContents: [ :str |
		str nextPutAll: '<html><head><title>'; nextPutAll: self realm; nextPutAll: '</title></head>'.
		str nextPutAll: '<body><h1>'; nextPutAll: self realm; nextPutAll: '</h1>'.
		str nextPutAll: '<ul>'.
		self mczEntries do: [ :each | 
			str nextPutAll: '<li><a href="'; nextPutAll: each; nextPutAll: '">'; nextPutAll: each; nextPutAll: '</a></li>'].
		str nextPutAll: '</ul></body></html>'.
		self mcmEntries do: [ :each | 
			str nextPutAll: '<li><a href="'; nextPutAll: each; nextPutAll: '">'; nextPutAll: each; nextPutAll: '</a></li>'].
		str nextPutAll: '</ul></body></html>'
		 ]! !

!MCWebServerSec methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 11:26'!
webServer
	^ webServer! !

!MCWebServerSec methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 14:45'!
webServer: aWebServer
	webServer := aWebServer! !
